@@ -8,6 +8,7 @@ import subprocess
 import csv
 from pySmartDL import SmartDL
 from pathlib import Path
+from threading import Thread
 from awcli.utilities import *
 
 def safeExit():
@@ -266,7 +267,51 @@ def addToCronologia(ep: int):
             log.insert(0, [anime.name, ep, anime.url]) 
 
 
-def openVideos(ep_iniziale: int, ep_finale: int, mpv: bool):
+def updateAnilist(tokenAnilist: str, ratingAnilist: bool, preferitoAnilist: bool,  ep: int):
+    """
+    Procede ad aggiornare l'anime su AniList.
+    Se l'episodio riprodotto è l'ultimo e
+    l'utente ha scelto di votare gli anime,
+    verrà chiesto il voto da dare.
+
+    Args:
+        tokenAnilist (str): il token di accesso ad AniList.
+        ratingAnilist (bool): valore impostato a True se l'utente ha scelto di votare l'anime una volta finito, altrimenti False.
+        preferitoAnilist(bool): True se l'utente ha scelto di far chiedere se mettere l'anime tra i preferiti, altrimenti False.
+        ep (int): il numero dell'episodio visualizzato.
+    """
+
+    voto = 0
+    preferiti = False
+    status_list = 'CURRENT'
+    #se ho finito di vedere l'anime
+    if ep == anime.ep and anime.status == 1:
+        status_list = 'COMPLETED'
+        #chiedo di votare
+        if ratingAnilist:
+            def is_number(n):
+                if n.isdigit():
+                    return n
+
+            voto = my_input("Inserisci un voto per l'anime", is_number)
+    
+            voto = float(voto)
+        #chiedo di mettere tra i preferiti
+        if preferitoAnilist:
+            def check_string(s: str):
+                s = s.lower()
+                if s == "s":
+                    return True
+                elif s == "n" or s == "":
+                    return False
+
+            preferiti = my_input("Mettere l'anime tra i preferiti? (s/N)", check_string)
+    
+    thread = Thread(target=anilistApi, args=(tokenAnilist, anime.id_anilist, ep, voto, status_list, preferiti))
+    thread.start()
+
+
+def openVideos(ep_iniziale: int, ep_finale: int, mpv: bool, tokenAnilist: str, ratingAnilist: bool, preferitoAnilist: bool):
     """
     Riproduce gli episodi dell'anime, a partire da ep_iniziale fino a ep_finale.
     Se un episodio è già stato scaricato, viene riprodotto dal file scaricato.
@@ -276,6 +321,9 @@ def openVideos(ep_iniziale: int, ep_finale: int, mpv: bool):
         ep_iniziale (int): il numero di episodio iniziale da riprodurre.
         ep_finale (int): il numero di episodio finale da riprodurre.
         mpv (bool): True se il player di default è MPV, False se è VLC.
+        tokenAnilist (str): il token di accesso ad AniList.
+        ratingAnilist (bool): valore impostato a True se l'utente ha scelto di votare l'anime una volta finito, altrimenti False.
+        preferitoAnilist(bool): True se l'utente ha scelto di far chiedere se mettere l'anime tra i preferiti, altrimenti False.
     """
 
     for ep in range(ep_iniziale, ep_finale+1):
@@ -295,10 +343,15 @@ def openVideos(ep_iniziale: int, ep_finale: int, mpv: bool):
 
         my_print(f"Riproduco {nome_video}...", color="giallo", cls=True)
         openMPV(url_server, nome_video) if mpv else openVLC(url_server, nome_video)
+
         #se non sono in modalità offline aggiungo l'anime alla cronologia
         if not offline:
             addToCronologia(ep)
 
+        #update watchlist anilist se ho fatto l'accesso
+        if tokenAnilist != 'tokenAnilist: False':
+            updateAnilist(tokenAnilist, ratingAnilist, preferitoAnilist, ep)
+
 
 def getCronologia() -> tuple[list, list]:
     """
@@ -321,19 +374,86 @@ def getCronologia() -> tuple[list, list]:
     return animes, episodi
 
 
-def getConfig() -> bool:
+def setupConfig() -> None:
     """
-    Prende il nome del player scelto dal file di configurazione.
+    Crea un file di configurazione chiamato "aw.config"
+    nella stessa directory dello script.
+    Le informazioni riportate saranno scelte dall'utente.
+    Sarà possibile scegliere il Player predefinito
+    e se collegare il proprio profilo AniList. 
+    """
+    try:
+        #player predefinito
+        my_print("", end="", cls=True)
+        my_print("AW-CLI - CONFIGURAZIONE", color="giallo")
+        my_print("1", color="verde", end="  ")
+        my_print("MPV")
+        my_print("2", color="verde", end="  ")
+        my_print("VLC")
+
+        def check_index(s: str):
+            if s == "1":
+                return "Player: MPV"
+            elif s == "2":
+                return "Player: VLC"
+
+        player = my_input("Scegli un player predefinito", check_index)
+
+        #animelist
+        def check_string(s: str):
+            s = s.lower()
+            if s == "s":
+                return True
+            elif s == "n" or s == "":
+                return False
+
+        anilist = my_input("Aggiornare automaticamente la watchlist con AniList? (s/N)", check_string)
+
+        if anilist:
+            if nome_os == "Linux" or nome_os == "Android":
+                os.system("xdg-open 'https://anilist.co/api/v2/oauth/authorize?client_id=11388&response_type=token' &>/dev/null")
+            elif nome_os == "Windows":
+                subprocess.Popen(['powershell.exe', "explorer https://anilist.co/api/v2/oauth/authorize?client_id=11388&response_type=token"])
+            tokenAnilist = my_input("Inserire il token di AniList", cls=True)
+            ratingAnilist = "ratingAnilist: True " if my_input("Votare l'anime una volta completato? (s/N)", check_string) else "ratingAnilist: False"
+            preferitoAnilist = "preferitoAnilist: True" if my_input("Chiedere se mettere l'anime tra i preferiti una volta completato? (s/N)", check_string) else "preferitoAnilist: False"
+        else:
+            tokenAnilist = "tokenAnilist: False"
+            ratingAnilist = "ratingAnilist: False"
+            preferitoAnilist = "preferitoAnilist: False"
+    except KeyboardInterrupt:
+        safeExit()
+    #creo il file
+    config = f"{os.path.dirname(__file__)}/aw.config"
+    with open(config, 'w') as config_file:
+        config_file.write(f"{player}\n")
+        config_file.write(f"{tokenAnilist}\n")
+        config_file.write(f"{ratingAnilist}\n")
+        config_file.write(preferitoAnilist)
+
+
+def getConfig() -> tuple[bool, str, bool, bool]:
+    """
+    Prende le impostazioni scelte dall'utente
+    dal file di configurazione.
 
     Returns:
-        bool: True se mpv è stato scelto, False se è stato scelto vlc.
+        tuple[bool, str, bool]: mpv ritorna True se è stato scelto mpv, altrimenti false se è VLC.
+        tokenAnilist ritorna il token di Anilist se è stato inserito. ratingAnilist ritorna True
+        se l'utente ha scelto di votare gli anime, altrimenti False. preferitoAnilist ritorna
+        True se l'utente ha scelto di chiedere se l'anime deve essere aggiunto tra i preferiti,
+        altrimenti False.
     """
 
     config = f"{os.path.dirname(__file__)}/aw.config"
     with open(config, 'r') as config_file:
-        for line in config_file:
-            mpv = True if line == 'Player: MPV' else False
-    return mpv
+        lines = config_file.readlines()
+        mpv = True if lines[0].strip() == "Player: MPV" else False
+        tokenAnilist = lines[1].strip()
+        ratingAnilist = True if lines[2].strip() == "ratingAnilist: True" else False
+        preferitoAnilist = True if lines[3].strip() == "preferitoAnilist: True" else False
+
+    return mpv, tokenAnilist, ratingAnilist, preferitoAnilist
 
 
 def main():
@@ -343,6 +463,7 @@ def main():
     global lista
     global offline
     global cronologia
+    global info
     global anime
     try:
         with open(f"{os.path.dirname(__file__)}/aw-cronologia.csv") as file:
@@ -351,21 +472,27 @@ def main():
         pass
     # args
     parser = argparse.ArgumentParser("aw-cli", description="Guarda anime dal terminale e molto altro!")
+    parser.add_argument('-a', '--configurazione', action='store_true', dest='avvia_config', help='avvia il menu di configurazione')
     parser.add_argument('-c', '--cronologia', action='store_true', dest='cronologia', help='continua a guardare un anime dalla cronologia')
     parser.add_argument('-d', '--download', action='store_true', dest='download', help='scarica gli episodi che preferisci')
+    parser.add_argument('-i', '--info', action='store_true', dest='info', help='visualizza le informazioni e la trama di un anime')
     parser.add_argument('-l', '--lista', nargs='?', choices=['a', 's', 'd'], dest='lista', help='lista degli ultimi anime usciti su AnimeWorld. a = all, s = sub, d = dub')
     parser.add_argument('-o', '--offline', action='store_true', dest='offline', help='apri gli episodi scaricati precedentemente direttamente dal terminale')
     if nome_os != "Android":
         parser.add_argument('-s', '--syncplay', action='store_true', dest='syncpl', help='usa syncplay per guardare un anime insieme ai tuoi amici')
     args = parser.parse_args()
 
-    
+
     if  '-l' in sys.argv and args.lista == None:
         args.lista = 'a'
 
     if nome_os != "Android":
         if args.syncpl:
             syncpl = True
+    if args.avvia_config:
+        setupConfig()
+    if args.info:
+        info = True
     if args.download:
         downl = True
     if args.lista:
@@ -375,7 +502,10 @@ def main():
     elif args.cronologia:
         cronologia = True
 
-    mpv = getConfig()
+    #se il file di configurazione non esiste viene chiesto all'utente di fare il setup
+    if not os.path.exists(f"{os.path.dirname(__file__)}/aw.config"):
+        setupConfig()
+    mpv, tokenAnilist, ratingAnilist, preferitoAnilist = getConfig()
 
     while True:
         try:
@@ -407,7 +537,12 @@ def main():
                 if lista:
                     ep_iniziale = anime.ep
                     ep_finale = ep_iniziale
-                anime.load_episodes() if not offline else anime.downloaded_episodes(f"{downloadPath()}/{anime.name}")
+                scelta_info = ""
+                if info:
+                    scelta_info = anime.getAnimeInfo()
+                    if scelta_info == 'i':
+                        break
+                anime.load_episodes(tokenAnilist) if not offline else anime.downloaded_episodes(f"{downloadPath()}/{anime.name}")
 
                 if anime.ep != 0:
                     break
@@ -415,7 +550,9 @@ def main():
                 # se l'anime non ha episodi non può essere selezionato
                 my_print("Eh, volevi! L'anime non è ancora stato rilasciato", color="rosso")
                 time.sleep(1)
-
+            #se ho l'args -i e ho scelto di tornare indietro, faccio una continue sul ciclo while True
+            if scelta_info == 'i':
+                continue
             if not cronologia:
                 if not lista:
                     ep_iniziale, ep_finale = scegliEpisodi()
@@ -444,13 +581,13 @@ def main():
                     
                     #chiedi all'utente se aprire ora i video scaricati
                     if my_input("Aprire ora il player con gli episodi scaricati? (S/n)", lambda i: i.lower()) in ['s', '']:
-                        openVideos(ep_iniziale, ep_finale, mpv)
+                        openVideos(ep_iniziale, ep_finale, mpv, tokenAnilist, ratingAnilist, preferitoAnilist)
                 safeExit()
 
             ris_valida = True
             while True:
                 if ris_valida:
-                    openVideos(ep_iniziale,ep_finale, mpv)
+                    openVideos(ep_iniziale,ep_finale, mpv, tokenAnilist, ratingAnilist, preferitoAnilist)
                 else:
                     my_print("Seleziona una risposta valida", color="rosso")
                     ris_valida = True
@@ -508,6 +645,7 @@ downl = False
 lista = False
 offline = False
 cronologia = False
+info = False
 log = []
 
 anime = Anime("", "")
